"""memory_abstract

迁移 ID: 1e67057a1646
父迁移: 8b619d7fca40
创建时间: 2026-01-02 18:53:23.916959

"""

from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "1e67057a1646"
down_revision: str | Sequence[str] | None = "8b619d7fca40"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("suggarchat_memory_data", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "memory_abstract",
                sa.Text(),
                nullable=False,
                server_default="",
            )
        )

    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("suggarchat_memory_data", schema=None) as batch_op:
        batch_op.drop_column(
            "memory_abstract",
        )

    # ### end Alembic commands ###
